// Copyright 2016 Michael Schenk. All rights reserved.
// Use of this source code is governed by a MIT-style
// license that can be found in the LICENSE file.

package command

import (
	"flag"
	"fmt"
	"os"
	"strings"

	"github.com/git-time-metric/gtm/metric"
	"github.com/git-time-metric/gtm/note"
	"github.com/git-time-metric/gtm/project"
	"github.com/git-time-metric/gtm/report"
	"github.com/git-time-metric/gtm/util"
	"github.com/mitchellh/cli"
)

// StatusCmd containt methods for status command
type StatusCmd struct {
}

// NewStatus returns new StatusCmd struct
func NewStatus() (cli.Command, error) {
	return StatusCmd{}, nil
}

// Help returns help for status command
func (r StatusCmd) Help() string {
	return r.Synopsis()
}

// Run executes status command with args
func (r StatusCmd) Run(args []string) int {
	statusFlags := flag.NewFlagSet("status", flag.ExitOnError)
	color := statusFlags.Bool(
		"color",
		false,
		"Always output color even if no terminal is detected. Use this with pagers i.e 'less -R' or 'more -R'")
	terminalOff := statusFlags.Bool(
		"terminal-off",
		false,
		"Exclude time spent in terminal (Terminal plugin is required)")
	totalOnly := statusFlags.Bool(
		"total-only",
		false,
		"Only display total time")
	tags := statusFlags.String(
		"tags",
		"",
		"Project tags to show status on")
	all := statusFlags.Bool(
		"all",
		false,
		"Show status for all projects")
	if err := statusFlags.Parse(os.Args[2:]); err != nil {
		fmt.Fprint(os.Stderr, err)
		return 1
	}

	if *totalOnly && (*all || *tags != "") {
		fmt.Fprint(os.Stderr, "\n-tags and -all options not allowed with -total-only\n")
		return 1
	}

	var (
		err        error
		commitNote note.CommitNote
		out        string
	)

	index, err := project.NewIndex()
	if err != nil {
		fmt.Fprintln(os.Stderr, err)
		return 1
	}

	tagList := []string{}
	if *tags != "" {
		tagList = util.Map(strings.Split(*tags, ","), strings.TrimSpace)
	}

	projects, err := index.Get(tagList, *all)
	if err != nil {
		fmt.Fprintln(os.Stderr, err)
		return 1
	}

	options := report.OutputOptions{
		TotalOnly:   *totalOnly,
		TerminalOff: *terminalOff,
		Color:       *color}

	for _, projPath := range projects {
		if commitNote, err = metric.Process(true, projPath); err != nil {
			fmt.Fprint(os.Stderr, err)
			return 1
		}
		o, err := report.Status(commitNote, options, projPath)
		if err != nil {
			fmt.Fprint(os.Stderr, err)
			return 1
		}
		out += o
	}

	fmt.Printf(out)
	return 0
}

// Synopsis returns help for status command
func (r StatusCmd) Synopsis() string {
	return `
	Usage: gtm status [-total-only] [-tags tag1,tag2] [-all] [-color] [-terminal-off] 
	Show time spent for working or staged files
	`
}
